#!/usr/bin/env python
# coding=utf-8

# Copyright (c) Mikhail Mamrouski.
# See LICENSE for details.

"""
This module provides a `Base` class used as a base class
for implementing MPRIS2 interfaces.

Decorator 'signal_wrapper' used by `Base` class
to convert function's arguments from dbus type to python type.

`BaseMeta` metaclass (inherited ExceptionMeta and ConverterMeta classes)
to avoid returning or raising dbus types and exceptions.
"""

from functools import partial, wraps
import types
import dbus
from common import convert, converter, exception_wrapper

IPROPERTIES = "org.freedesktop.DBus.Properties"


def signal_wrapper(f):
    """Decorator converts function's arguments from dbus types to python."""
    @wraps(f)
    def wrapper(*args, **kwds):
        args = map(convert, args)
        kwds = {convert(k): convert(v) for k, v in kwds.items()}
        return f(*args, **kwds)
    return wrapper


class ExceptionMeta(type):
    """Metaclass wraps all class' functions and properties
    in `exception_wrapper` decorator to avoid raising dbus exceptions
    """
    def __new__(cls, name, parents, dct):
        fn = exception_wrapper
        for attr_name in dct:
            if isinstance(dct[attr_name], types.FunctionType):
                dct[attr_name] = fn(dct[attr_name])
            elif isinstance(dct[attr_name], property) and dct[attr_name].fget:
                dct[attr_name] = property(
                    fn(dct[attr_name].fget) if dct[attr_name].fget else None,
                    fn(dct[attr_name].fset) if dct[attr_name].fset else None,
                    fn(dct[attr_name].fdel) if dct[attr_name].fdel else None)

        return super(ExceptionMeta, cls).__new__(cls, name, parents, dct)


class ConverterMeta(type):
    """Metaclass wraps all class' functions and properties
    in `converter` decorator to avoid returning dbus types
    """

    def __new__(cls, name, parents, dct):
        for attr_name in dct:
            if isinstance(dct[attr_name], types.FunctionType):
                dct[attr_name] = converter(dct[attr_name])
            elif isinstance(dct[attr_name], property) and dct[attr_name].fget:
                dct[attr_name] = property(converter(dct[attr_name].fget),
                                          dct[attr_name].fset,
                                          dct[attr_name].fdel
                                          )

        return super(ConverterMeta, cls).__new__(cls, name, parents, dct)


class BaseMeta(ExceptionMeta, ConverterMeta):
    pass


class Base(object):

    """Base class provides common functionality
    for other classes which implement MPRIS2 interfaces"""

    __metaclass__ = BaseMeta
    OBJ_PATH = "/org/mpris/MediaPlayer2"

    def __init__(self, name, bus=None, private=False):
        """Init inner attributes to work with dbus.
        Parameters:
            name - unique or well-known objects name
            bus - bus object;
                  new SessionBus() object will be created if value is None.
            private - private connection (uses only if bus is None)

        below described attributes will be created:
            bus - Bus object from the functions argument or SessionBus()
            name - objects name from the functions argument
            proxy - DBUS proxy object
            iface - DBUS interface (uses self.IFACE path to create it)
            properties - DBUS interface to work with object's properties
            get - function to receive property's value
            set - function to set property's value
        """
        if not bus:
            bus = dbus.SessionBus(private=private)
        self.bus = bus
        self.name = name
        self.proxy = bus.get_object(name, self.OBJ_PATH)
        self.iface = dbus.Interface(self.proxy, self.IFACE)
        self.properties = dbus.Interface(self.proxy, IPROPERTIES)
        self.get = partial(self.properties.Get, self.IFACE)
        self.set = partial(self.properties.Set, self.IFACE)

    def register_signal_handler(self, signal_name, handler_function):
        """register `handler_function` to receive `signal_name`.

        Uses class's dbus interface self.IFACE, objects name self.name
            and objects path self.OBJ_PATH to match signal.
        """
        self.bus.add_signal_receiver(signal_wrapper(handler_function),
                                     signal_name=signal_name,
                                     dbus_interface=self.IFACE,
                                     bus_name=self.name,
                                     path=self.OBJ_PATH)



#!/usr/bin/env python
# coding=utf-8

# Copyright (c) Mikhail Mamrouski.
# See LICENSE for details.

"""
This module provides a `TrackList` class
wich implemented MPRIS2 TrackList interface:
http://specifications.freedesktop.org/mpris-spec/latest/Track_List_Interface.html

Usage:

tl = TrackList('org.mpris.MediaPlayer2.vlc')
print tl.Tracks
tl.RemoveTrack(tl.Tracks[2])

"""

from common import convert2dbus
from Base import Base


class TrackList(Base):

    """class implements methods and properties
    to working with MPRIS2 TrackList interface
    """

    IFACE = "org.mpris.MediaPlayer2.TrackList"

    def __init__(self, name, bus=None, private=False):
        super(TrackList, self).__init__(name, bus, private)

    def GetTracksMetadata(self, track_ids):
        """Gets all the metadata available for a set of tracks.
        Parameters:
            track_ids - list of track ids

        Returns:
            Metadata of the set of tracks given as input.
        """
        return self.iface.GetTracksMetadata(convert2dbus(track_ids, 'ao'))

    def AddTrack(self, uri, after_track, set_as_current):
        """Adds a URI in the TrackList.
        Parameters:
            uri — The uri of the item to add.
            after_track — The identifier of the track
                          after which the new item should be inserted.
            set_as_current - Whether the newly inserted track
                             should be considered as the current track.
        """
        self.iface.AddTrack(uri,
                            convert2dbus(after_track, 'o'),
                            convert2dbus(set_as_current, 'b'))

    def RemoveTrack(self, track_id):
        """Removes an item from the TrackList.
        Parameters:
            track_id - Identifier of the track to be removed.
        """
        self.iface.RemoveTrack(convert2dbus(track_id, 'o'))

    def GoTo(self, track_id):
        """Skip to the specified TrackId.
        Parameters:
            track_id - Identifier of the track to skip to.
        """
        self.iface.GoTo(convert2dbus(track_id, 'o'))

    @property
    def Tracks(self):
        """Returns an list which contains the identifier of each track
        in the tracklist, in order."""
        return self.get('Tracks')

    @property
    def CanEditTracks(self):
        """If false, calling AddTrack or RemoveTrack will have no effect,
        and may raise a NotSupported error."""
        return self.get('CanEditTracks')



#!/usr/bin/env python
# coding=utf-8

# Copyright (c) Mikhail Mamrouski.
# See LICENSE for details.

"""
This module provides a `Root` class
wich implemented MPRIS2 Root interface:
http://specifications.freedesktop.org/mpris-spec/latest/Media_Player.html

Usage:

root = Root('org.mpris.MediaPlayer2.vlc')
print root.Identity
if root.CanRaise:
    root.Raise()
print "Supported Mime Types: ", root.SupportedMimeTypes
print "Supported Uri Schemes: ", root.SupportedUriSchemes

if root.CanQuit:
    root.Quit
"""

from Base import Base


class Root(Base):

    """class implements methods and properties
    to working with MPRIS2 Root interface
    """

    IFACE = "org.mpris.MediaPlayer2"

    def __init__(self, name, bus=None, private=False):
        super(Root, self).__init__(name, bus, private)

    def Raise(self):
        """Brings the media player's user interface to the front
        using any appropriate mechanism available.
        The media player may be unable to control how its user interface
        is displayed, or it may not have a graphical user interface at all.
        In this case, the CanRaise property is false
        and this method does nothing."""
        self.iface.Raise()

    def Quit(self):
        """Causes the media player to stop running.
        The media player may refuse to allow clients to shut it down.
        In this case, the CanQuit property is false
        and this method does nothing.
        Note: Media players which can be D-Bus activated,
        or for which there is no sensibly easy way to terminate
        a running instance (via the main interface or a notification area icon
        for example) should allow clients to use this method.
        Otherwise, it should not be needed.
        If the media player does not have a UI, this should be implemented."""
        self.iface.Quit()

    @property
    def CanQuit(self):
        """If false, calling Quit will have no effect,
        and may raise a NotSupported error.
        If true, calling Quit will cause the media application to attempt
        to quit (although it may still be prevented from quitting by the user,
        for example)."""
        return self.get('CanQuit')

    @property
    def Fullscreen(self):
        """This property is optional.
        Clients should handle its absence gracefully.
        Whether the media player is occupying the fullscreen.
        This is typically used for videos. A value of true indicates that
        the media player is taking up the full screen.
        Media centre software may well have this value fixed to true
        If CanSetFullscreen is true, clients may set this property to true
        to tell the media player to enter fullscreen mode,
        or to false to return to windowed mode.
        If CanSetFullscreen is false, then attempting to set this property
        should have no effect, and may raise an error.
        However, even if it is true, the media player may still be unable
        to fulfil the request, in which case attempting to set this property
        will have no effect (but should not raise an error)."""
        return self.get('Fullscreen')

    @Fullscreen.setter
    def Fullscreen(self, state):
        """Set Fullscreen property"""
        self.set('Fullscreen', state)

    @property
    def CanSetFullscreen(self):
        """If false, attempting to set Fullscreen will have no effect,
        and may raise an error.
        If true, attempting to set Fullscreen will not raise an error,
        and (if it is different from the current value) will cause
        the media player to attempt to enter or exit fullscreen mode.
        Note that the media player may be unable to fulfil the request.
        In this case, the value will not change.
        If the media player knows in advance that it will not be able
        to fulfil the request, however, this property should be false."""
        return self.get('CanSetFullscreen')

    @property
    def CanRaise(self):
        """If false, calling Raise will have no effect,
        and may raise a NotSupported error.
        If true, calling Raise will cause the media application to attempt
        to bring its user interface to the front,
        although it may be prevented from doing so
        (by the window manager, for example). """
        return self.get('CanRaise')

    @property
    def HasTrackList(self):
        """Indicates whether the /org/mpris/MediaPlayer2 object
        implements the org.mpris.MediaPlayer2.TrackList interface."""
        return self.get('HasTrackList')

    @property
    def Identity(self):
        """A friendly name to identify the media player to users.
        This should usually match the name found in .desktop files
        (eg: "VLC media player")."""
        return self.get('Identity')

    @property
    def DesktopEntry(self):
        """The basename of an installed .desktop file which complies
        with the Desktop entry specification,
        with the ".desktop" extension stripped.
        Example:
            The desktop entry file is "/usr/share/applications/vlc.desktop",
            and this property contains "vlc"
        """
        return self.get('DesktopEntry')

    @property
    def SupportedUriSchemes(self):
        """The URI schemes supported by the media player.
        This can be viewed as protocols supported by the player
        in almost all cases. Almost every media player will include support
        for the "file" scheme. Other common schemes are "http" and "rtsp".
        Note that URI schemes should be lower-case.
        """
        return self.get('SupportedUriSchemes')

    @property
    def SupportedMimeTypes(self):
        """The mime-types supported by the media player.
        Mime-types should be in the standard format
            (eg: audio/mpeg or application/ogg).
        """
        return self.get('SupportedMimeTypes')



from MediaPlayer import MediaPlayer
from PlayLists import PlayLists, PlaylistOrdering
from Player import Player
from Root import Root
from TrackList import TrackList
from common import available_players, PyMPRISException

__version__ = '1.2'
__description__ = 'Library to control media players using MPRIS2 interfaces'
requires = [
    # pympris depends on dbus-python,
    # but dbus-python can't be installed automaticaly
    # 'dbus-python>=1.0'
]
README = """pympris is a Python library used
to control media players using MPRIS2 interfaces.

Usage
=====

::

import gobject
import dbus
from dbus.mainloop.glib import DBusGMainLoop

import pympris

dbus_loop = DBusGMainLoop()
bus = dbus.SessionBus(mainloop=dbus_loop)

# get unique ids for all available players
players_ids = list(pympris.available_players())
mp = pympris.MediaPlayer(players_ids[1], bus)

# mp.root implements org.mpris.MediaPlayer2 interface
# mp.player implements org.mpris.MediaPlayer2.Player
# mp.track_list implements org.mpris.MediaPlayer2.TrackList
# mp.playlists implements org.mpris.MediaPlayer2.Playlists

# print player Identity
print mp.root.Identity

if mp.root.CanRaise:
    mp.root.Raise()

if mp.player.CanPlay and mp.player.CanPause:
    mp.player.PlayPause()

mp.player.Volume = mp.player.Volume*2

if mp.player.CanGoNext:
    mp.player.Next()

tracks = mp.track_list.Tracks
for track_id in tracks:
    print track_id

if len(tracks) > 1:
    mp.track_list.RemoveTrack(tracks[-1])
    mp.track_list.GoTo(tracks[0])

n = mp.playlists.PlaylistCount
ordering = pympris.PlaylistOrdering.LastPlayDate
playlists = mp.playlists.GetPlaylists(0, n, ordering, reversed=False)
pl_id, pl_name, pl_icon = playlists[-2]
mp.playlists.ActivatePlaylist(pl_id)

# setup signal handlers


def seeked(x):
    print(x)


def PlaylistChanged(arg):
    print "PlaylistChanged", arg


def TrackMetadataChanged(track_id, metadata):
    print "TrackMetadataChanged", track_id, metadata


def TrackListReplaced(tracks, current_track):
    print "TrackListReplaced", tracks, current_track


def TrackAdded(metadata, after_track):
    print "TrackAdded", metadata, after_track


def TrackRemoved(track_id):
    print "TrackRemoved", track_id


mp.player.register_signal_handler('Seeked', seeked)
mp.playlists.register_signal_handler('PlaylistChanged', PlaylistChanged)
mp.track_list.register_signal_handler('TrackMetadataChanged',
                                      TrackMetadataChanged)
mp.track_list.register_signal_handler('TrackListReplaced', TrackListReplaced)
mp.track_list.register_signal_handler('TrackAdded', TrackAdded)
mp.track_list.register_signal_handler('TrackRemoved', TrackRemoved)

loop = gobject.MainLoop()
loop.run()

"""



#!/usr/bin/env python
# coding=utf-8

# Copyright (c) Mikhail Mamrouski.
# See LICENSE for details.

"""
This module provides helper functions:

`convert2dbus` - converts from Python type to DBUS type according signature
`convert` - function to convert dbus object to python object.
`converter` - decorator to convert dbus object to python object.
`exception_wrapper` - decorator to convert dbus exception to pympris exception.
`available_players` - function searchs and returns unique names of objects
                      which implemented MPRIS2 interfaces.

"""

from functools import wraps
import dbus

MPRIS_NAME_PREFIX = "org.mpris.MediaPlayer2"


def convert2dbus(value, signature):
    """Convert Python type to dbus type according signature"""
    if len(signature) == 2 and signature.startswith('a'):
        return dbus.Array(value, signature=signature[-1])
    type_map = {
        'b': dbus.Boolean, 'y': dbus.Byte, 'n': dbus.Int16,
        'i': dbus.Int32, 'x': dbus.Int64, 'q': dbus.UInt16, 'u': dbus.UInt32,
        't': dbus.UInt64, 'd': dbus.Double, 'o': dbus.ObjectPath,
        'g': dbus.Signature, 's': dbus.UTF8String}
    return type_map[signature](value)


def convert(dbus_obj):
    if isinstance(dbus_obj, dbus.Boolean):
        return bool(dbus_obj)
    if filter(lambda obj_type: isinstance(dbus_obj, obj_type),
              (dbus.Byte, dbus.Int16, dbus.Int32, dbus.Int64,
               dbus.UInt16, dbus.UInt32, dbus.UInt64)):
        return int(dbus_obj)
    if isinstance(dbus_obj, dbus.Double):
        return float(dbus_obj)
    if filter(lambda obj_type: isinstance(dbus_obj, obj_type),
             (dbus.ObjectPath, dbus.Signature, dbus.String, dbus.UTF8String)):
        return unicode(dbus_obj)
    if isinstance(dbus_obj, dbus.Array):
        return map(convert, dbus_obj)
    if isinstance(dbus_obj, dbus.Dictionary):
        return {convert(key): convert(value)
                for key, value in dbus_obj.items()}
    if isinstance(dbus_obj, dbus.Struct):
        return tuple(map(convert, dbus_obj))
    return dbus_obj


def converter(f):
    """Decorator to convert from dbus type to Python type"""
    @wraps(f)
    def wrapper(*args, **kwds):
        return convert(f(*args, **kwds))
    return wrapper


def exception_wrapper(f):
    """Decorator to convert dbus exception to pympris exception"""
    @wraps(f)
    def wrapper(*args, **kwds):
        try:
            return f(*args, **kwds)
        except dbus.exceptions.DBusException as err:
            _args = err.args
            raise PyMPRISException(*_args)
    return wrapper


def available_players():
    """Search and return set of unique names of objects
    which implemented MPRIS2 interfaces."""
    bus = dbus.SessionBus()
    players = set()
    for name in filter(lambda item: item.startswith(MPRIS_NAME_PREFIX),
                       bus.list_names()):
        owner_name = bus.get_name_owner(name)
        players.add(convert(owner_name))
    return players


class PyMPRISException(Exception):
    """docstring for PyMPRISException"""
    def __init__(self, *args):
        super(PyMPRISException, self).__init__(*args)



#!/usr/bin/env python
# coding=utf-8

# Copyright (c) Mikhail Mamrouski.
# See LICENSE for details.

"""
This module provides a `PlayLists` class
wich implemented MPRIS2 PlayLists interface:
http://specifications.freedesktop.org/mpris-spec/latest/Playlists_Interface.html

class PlaylistOrdering uses as an enum for Ordering type.

pl = PlayLists('org.mpris.MediaPlayer2.rhythmbox')
print pl.PlaylistCount
print pl.ActivePlaylist

items = pl.GetPlaylists(0, 100, PlaylistOrdering.Alphabetical, reversed=False)
for uri, name, icon_uri in items:
    print uri, name, icon_uri
"""

from common import convert2dbus
from Base import Base


class PlaylistOrdering(object):
    Alphabetical = 'Alphabetical'
    CreationDate = 'Created'
    ModifiedDate = 'Modified'
    LastPlayDate = 'Played'
    UserDefined = 'User'


class PlayLists(Base):

    """class implements methods and properties
    to working with MPRIS2 Playlists interface
    """

    IFACE = "org.mpris.MediaPlayer2.Playlists"

    def __init__(self, name, bus=None, private=False):
        super(PlayLists, self).__init__(name, bus, private)

    def ActivatePlaylist(self, playlist_id):
        """Starts playing the given playlist.
        Parameters:
            playlist_id - The id of the playlist to activate.
        """
        self.iface.ActivatePlaylist(playlist_id)

    def GetPlaylists(self, start, max_count, order, reversed):
        """Gets a set of playlists.
        Parameters:
            start - The index of the first playlist to be fetched
                    (according to the ordering).
            max_count - The maximum number of playlists to fetch.
            order - The ordering that should be used.
            reversed - Whether the order should be reversed.

        """
        cv = convert2dbus
        return self.iface.GetPlaylists(cv(start, 'u'),
                                       cv(max_count, 'u'),
                                       cv(order, 's'),
                                       cv(reversed, 'b'))

    @property
    def PlaylistCount(self):
        """The number of playlists available."""
        return self.get('PlaylistCount')

    @property
    def Orderings(self):
        """The available orderings. At least one must be offered."""
        return self.get('Orderings')

    @property
    def ActivePlaylist(self):
        """The currently-active playlist."""
        valid, info = tuple(self.get('ActivePlaylist'))
        if valid:
            return info



#!/usr/bin/env python
# coding=utf-8

# Copyright (c) Mikhail Mamrouski.
# See LICENSE for details.

"""
This module provides a `MediaPlayer` class
wich contains instances of all implementations of MPRIS2 interfaces.

Usage:

mp = MediaPlayer('org.mpris.MediaPlayer2.rhythmbox')
print mp.root.Identity
if mp.root.CanRaise:
    mp.root.Raise()

if mp.player.CanPause and mp.player.CanPlay:
    mp.player.PlayPause()
if mp.player.CanGoNext:
    mp.player.Next()

print mp.track_list.Tracks
print mp.playlists.GetPlaylists

if mp.root.CanQuit:
    mp.root.Quit()
"""

from Root import Root
from Player import Player
from PlayLists import PlayLists
from TrackList import TrackList


class MediaPlayer(object):

    """Class uses as helper class."""

    def __init__(self, dbus_name, bus=None, private=False):
        super(MediaPlayer, self).__init__()
        self.root = Root(dbus_name, bus, private)
        self.player = Player(dbus_name, bus, private)
        self.playlists = PlayLists(dbus_name, bus, private)
        self.track_list = TrackList(dbus_name, bus, private)



#!/usr/bin/env python
# coding=utf-8

# Copyright (c) Mikhail Mamrouski.
# See LICENSE for details.

"""
This module provides a `Player` class
wich implemented MPRIS2 Player interface:
http://specifications.freedesktop.org/mpris-spec/latest/Player_Interface.html

Usage:

player = Player('org.mpris.MediaPlayer2.vlc')
if player.CanPause:
    player.PlayPause()

player.Volume = player.Volume*2
player.Play()
if player.CanGoNext:
    player.Next()

if player.CanSeek:
    player.Seek = 15000000

"""

from common import convert2dbus
from Base import Base


class Player(Base):

    """class implements methods and properties
    to working with MPRIS2 Player interface
    """

    # __metaclass__ = ConverterMeta

    IFACE = "org.mpris.MediaPlayer2.Player"

    def __init__(self, name, bus=None, private=False):
        super(Player, self).__init__(name, bus, private)

    def Next(self):
        """Skips to the next track in the tracklist."""
        self.iface.Next()

    def Previous(self):
        """Skips to the previous track in the tracklist."""
        self.iface.Previous()

    def Pause(self):
        """Pauses playback."""
        self.iface.Pause()

    def PlayPause(self):
        """Pauses playback."""
        self.iface.PlayPause()

    def Stop(self):
        """Stops playback."""
        self.iface.Stop()

    def Play(self):
        """Starts or resumes playback."""
        self.iface.Play()

    def Seek(self, offset):
        """Seeks forward in the current track
        Parameters
            Offset — The number of microseconds to seek forward.
        A negative value seeks back.
        """
        self.iface.Seek(convert2dbus(offset, 'x'))

    def SetPosition(self, track_id, position):
        """Sets the current track position in microseconds.
        Parameters
            track_id - The currently playing track's identifier.
            position — Track position in microseconds.
                This must be between 0 and <track_length>.

        If the Position argument is less than 0, do nothing.
        If the Position argument is greater than the track length, do nothing.
        If the CanSeek property is false, this has no effect.
        """
        self.iface.SetPosition(convert2dbus(track_id, 'o'),
                               convert2dbus(position, 'x'))

    def OpenUri(self, uri):
        """Opens the Uri given as an argument
        Parameters
            uri - Uri of the track to load.
        Its uri scheme should be an element
        of the org.mpris.MediaPlayer2.SupportedUriSchemes property
        and the mime-type should match one of the elements
        of the org.mpris.MediaPlayer2.SupportedMimeTypes.

        If the playback is stopped, starts playing
        If the uri scheme or the mime-type of the uri to open is not supported,
        this method does nothing and may raise an error.
        """
        self.iface.OpenUri(uri)

    @property
    def PlaybackStatus(self):
        """The current playback status.
        May be "Playing", "Paused" or "Stopped".
        """
        return self.get('PlaybackStatus')

    @property
    def LoopStatus(self):
        """The current loop / repeat status
        May be:
            "None" if the playback will stop
                   when there are no more tracks to play
            "Track" if the current track will start again from
                    the begining once it has finished playing
            "Playlist" if the playback loops through a list of tracks
        """
        return self.get('LoopStatus')

    @LoopStatus.setter
    def LoopStatus(self, status):
        """The current loop / repeat status
        May be:
            "None" if the playback will stop
                   when there are no more tracks to play
            "Track" if the current track will start again from
                    the begining once it has finished playing
            "Playlist" if the playback loops through a list of tracks
        """
        self.set('LoopStatus', status)

    @property
    def Rate(self):
        """The current playback rate."""
        return self.get('Rate')

    @Rate.setter
    def Rate(self, value):
        """The current playback rate."""
        self.set('Rate', value)

    @property
    def Shuffle(self):
        """A value of false indicates that playback
        is progressing linearly through a playlist, while true means playback
        is progressing through a playlist in some other order.
        """
        return self.get('Shuffle')

    @Shuffle.setter
    def Shuffle(self, value):
        """A value of false indicates that playback
        is progressing linearly through a playlist, while true means playback
        is progressing through a playlist in some other order.
        """
        self.set('Shuffle', value)

    @property
    def Metadata(self):
        """The metadata of the current element."""
        return self.get('Metadata')

    @property
    def Volume(self):
        """The volume level"""
        return self.get('Volume')

    @Volume.setter
    def Volume(self, value):
        """The volume level"""
        self.set('Volume', value)

    @property
    def Position(self):
        """The current track position in microseconds,
        between 0 and the 'mpris:length' metadata entry (see Metadata).
        """
        return self.get('Position')

    @property
    def MinimumRate(self):
        """The minimum value which the Rate property can take.
        This value should always be 1.0 or less.
        """
        return self.get('MinimumRate')

    @property
    def MaximumRate(self):
        """The maximum value which the Rate property can take.
        This value should always be 1.0 or greater."""
        return self.get('MaximumRate')

    @property
    def CanGoNext(self):
        """Whether the client can call the Next method on this interface
        and expect the current track to change.
        """
        return self.get('CanGoNext')

    @property
    def CanGoPrevious(self):
        """Whether the client can call the Previous method on this interface
        and expect the current track to change.
        """
        return self.get('CanGoPrevious')

    @property
    def CanPlay(self):
        """Whether playback can be started using Play or PlayPause."""
        return self.get('CanPlay')

    @property
    def CanPause(self):
        """Whether playback can be paused using Pause or PlayPause."""
        return self.get('CanPause')

    @property
    def CanSeek(self):
        """Whether the client can control the playback position
        using Seek and SetPosition. This may be different for different tracks.
        """
        return self.get('CanSeek')

    @property
    def CanControl(self):
        """Whether the media player may be controlled over this interface."""
        return self.get('CanControl')



#!/usr/bin/env python
# coding=utf-8

from setuptools import setup
from pympris import __version__, __description__, requires, README


setup(name='pympris',
      version=__version__,
      description=__description__,
      author='Mikhail Mamrouski',
      author_email='wst.public.mail@gmail.com',
      url="https://bitbucket.org/wistful/pympris",
      license="MIT License",
      packages=['pympris'],
      long_description=README,
      install_requires=requires,
      platforms=["Unix,"],
      keywords="mpris, dbus",
      classifiers=[
          "Development Status :: 5 - Production/Stable",
          "Environment :: X11 Applications",
          "Intended Audience :: Developers",
          "License :: OSI Approved :: MIT License",
          "Programming Language :: Python :: 2.7",
          "Topic :: Software Development :: Libraries :: Python Modules",
      ],
      )
